@@ -71,6 +71,7 @@ class EventManager:
         self._registerListener("window:deactivate")
         self._registerListener("object:children-changed")
         self._registerListener("mouse:button")
+        self.registerKeystrokeListener(self._processKeyboardEvent)
         self._active = True
         debug.println(debug.LEVEL_INFO, 'EVENT MANAGER: Activated', True)
 
@@ -79,9 +80,10 @@ class EventManager:
 
         debug.println(debug.LEVEL_INFO, 'EVENT MANAGER: Dectivating', True)
         self._active = False
-        for eventType in list(self._scriptListenerCounts.keys()):
+        for eventType in self._scriptListenerCounts.keys():
             self.registry.deregisterEventListener(self._enqueue, eventType)
         self._scriptListenerCounts = {}
+        self.deregisterKeystrokeListener(self._processKeyboardEvent)
         debug.println(debug.LEVEL_INFO, 'EVENT MANAGER: Deactivated', True)
 
     def ignoreEventTypes(self, eventTypeList):
@@ -163,6 +165,14 @@ class EventManager:
             debug.println(debug.LEVEL_INFO, msg, True)
             return True
 
+        if event.type.startswith('object:state-changed:sensitive'):
+            if role in [pyatspi.ROLE_MENU_ITEM,
+                        pyatspi.ROLE_CHECK_MENU_ITEM,
+                        pyatspi.ROLE_RADIO_MENU_ITEM]:
+                msg = 'EVENT MANAGER: Ignoring event type due to role'
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return True
+
         if event.type.startswith('object:state-changed:showing'):
             if role not in [pyatspi.ROLE_ALERT,
                             pyatspi.ROLE_ANIMATION,
@@ -420,7 +430,7 @@ class EventManager:
         msg = 'EVENT MANAGER: registering listeners for: %s' % script
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        for eventType in list(script.listeners.keys()):
+        for eventType in script.listeners.keys():
             self._registerListener(eventType)
 
     def deregisterScriptListeners(self, script):
@@ -434,19 +444,19 @@ class EventManager:
         msg = 'EVENT MANAGER: deregistering listeners for: %s' % script
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        for eventType in list(script.listeners.keys()):
+        for eventType in script.listeners.keys():
             self._deregisterListener(eventType)
 
     def registerModuleListeners(self, listeners):
         """Register the listeners on behalf of the caller."""
 
-        for eventType, function in list(listeners.items()):
+        for eventType, function in listeners.items():
             self.registry.registerEventListener(function, eventType)
 
     def deregisterModuleListeners(self, listeners):
         """Deegister the listeners on behalf of the caller."""
 
-        for eventType, function in list(listeners.items()):
+        for eventType, function in listeners.items():
             self.registry.deregisterEventListener(function, eventType)
 
     def registerKeystrokeListener(self, function, mask=None, kind=None):
@@ -455,10 +465,10 @@ class EventManager:
         msg = 'EVENT MANAGER: registering keystroke listener function: %s' % function
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        if mask == None:
+        if mask is None:
             mask = list(range(256))
 
-        if kind == None:
+        if kind is None:
             kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
 
         self.registry.registerKeystrokeListener(function, mask=mask, kind=kind)
@@ -469,10 +479,10 @@ class EventManager:
         msg = 'EVENT MANAGER: deregistering keystroke listener function: %s' % function
         debug.println(debug.LEVEL_INFO, msg, True)
 
-        if mask == None:
+        if mask is None:
             mask = list(range(256))
 
-        if kind == None:
+        if kind is None:
             kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
 
         self.registry.deregisterKeystrokeListener(
@@ -489,10 +499,7 @@ class EventManager:
         if not orca_state.activeScript:
             return
 
-        if isinstance(event, input_event.KeyboardEvent):
-            function = orca_state.activeScript.processKeyboardEvent
-            data = "'%s' (%d)" % (event.event_string, event.hw_code)
-        elif isinstance(event, input_event.BrailleEvent):
+        if isinstance(event, input_event.BrailleEvent):
             function = orca_state.activeScript.processBrailleEvent
             data = "'%s'" % repr(event.event)
         else:
@@ -681,26 +688,18 @@ class EventManager:
             debug.println(debug.LEVEL_INFO, msg, True)
             debug.printException(debug.LEVEL_INFO)
 
-    def processKeyboardEvent(self, keyboardEvent):
-        """Processes the given keyboard event based on the keybinding from the
-        currently active script. This method is called synchronously from the
-        at-spi registry and should be performant.  In addition, it must return
-        True if it has consumed the event (and False if not).
+    def _processKeyboardEvent(self, event):
+        keyboardEvent = input_event.KeyboardEvent(event)
+        if not keyboardEvent.is_duplicate:
+            debug.println(debug.LEVEL_INFO, "\n%s" % keyboardEvent)
 
-        Arguments:
-        - keyboardEvent: an instance of input_event.KeyboardEvent
+        rv = keyboardEvent.process()
 
-        Returns True if the event should be consumed.
-        """
+        # Do any needed xmodmap crap. Hopefully this can die soon.
+        from orca import orca
+        orca.updateKeyMap(keyboardEvent)
 
-        consume = False
-        if orca_state.activeScript \
-           and orca_state.activeScript.consumesKeyboardEvent(keyboardEvent):
-            consume = not orca_state.bypassNextCommand
-            if consume:
-                self._enqueue(keyboardEvent)
-
-        return consume
+        return rv
 
     def processBrailleEvent(self, brailleEvent):
         """Called whenever a cursor key is pressed on the Braille display.
