@@ -42,13 +42,13 @@ import orca.orca_state as orca_state
 import orca.speech as speech
 import orca.settings as settings
 import orca.settings_manager as settings_manager
+import orca.structural_navigation as structural_navigation
 
 from .braille_generator import BrailleGenerator
 from .formatting import Formatting
 from .script_utilities import Utilities
 from .spellcheck import SpellCheck
 from .speech_generator import SpeechGenerator
-from .structural_navigation import StructuralNavigation
 
 _settingsManager = settings_manager.getManager()
 
@@ -106,14 +106,14 @@ class Script(default.Script):
         """Returns the 'structural navigation' class for this script.
         """
         types = self.getEnabledStructuralNavigationTypes()
-        return StructuralNavigation(self, types, enabled=False)
+        return structural_navigation.StructuralNavigation(self, types, enabled=False)
 
     def getEnabledStructuralNavigationTypes(self):
         """Returns a list of the structural navigation object types
         enabled in this script.
         """
 
-        enabledTypes = [StructuralNavigation.TABLE_CELL]
+        enabledTypes = [structural_navigation.StructuralNavigation.TABLE_CELL]
 
         return enabledTypes
 
@@ -290,27 +290,6 @@ class Script(default.Script):
         prefs.update(self.spellcheck.getPreferencesFromGUI())
         return prefs
 
-    def isStructuralNavigationCommand(self, inputEvent=None):
-        """Checks to see if the inputEvent was a structural navigation
-        command. This is necessary to prevent double-presentation of
-        items. [[[TODO - JD: Perhaps this should be moved to default.py]]]
-
-        Arguments:
-        - inputEvent: The input event to examine. If none is provided,
-          orca_state.lastInputEvent will be used.
-
-        Returns True if inputEvent is a structural navigation command
-        enabled in this script.
-        """
-
-        inputEvent = inputEvent or orca_state.lastInputEvent
-        if isinstance(inputEvent, input_event.KeyboardEvent):
-            if self.structuralNavigation.keyBindings.\
-                    getInputHandler(inputEvent):
-                return True
-
-        return False
-
     def doWhereAmI(self, inputEvent, basicOnly):
         """Performs the whereAmI operation."""
 
@@ -319,6 +298,14 @@ class Script(default.Script):
 
         super().doWhereAmI(inputEvent, basicOnly)
 
+    def presentObject(self, obj, **args):
+        if not self._lastCommandWasStructNav:
+            super().presentObject(obj, **args)
+            return
+
+        utterances = self.speechGenerator.generateSpeech(obj, **args)
+        speech.speak(utterances)
+
     def panBrailleLeft(self, inputEvent=None, panAmount=0):
         """In document content, we want to use the panning keys to browse the
         entire document.
@@ -595,10 +582,16 @@ class Script(default.Script):
         # Pass the event onto the parent class to be handled in the default way.
         default.Script.locusOfFocusChanged(self, event,
                                            oldLocusOfFocus, newLocusOfFocus)
+        if not newLocusOfFocus:
+            return
 
-        if self.utilities.isDocumentCell(newLocusOfFocus):
-            row, column, table = \
-                self.utilities.getRowColumnAndTable(newLocusOfFocus.parent)
+        cell = None
+        if self.utilities.isTextDocumentCell(newLocusOfFocus):
+            cell = newLocusOfFocus
+        elif self.utilities.isTextDocumentCell(newLocusOfFocus.parent):
+            cell = newLocusOfFocus.parent
+        if cell:
+            row, column = self.utilities.coordinatesForCell(cell)
             self.pointOfReference['lastRow'] = row
             self.pointOfReference['lastColumn'] = column
 
@@ -631,6 +624,11 @@ class Script(default.Script):
     def onActiveChanged(self, event):
         """Callback for object:state-changed:active accessibility events."""
 
+        if not event.source.parent:
+            msg = "SOFFICE: Event source lacks parent"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+
         # Prevent this events from activating the find operation.
         # See comment #18 of bug #354463.
         if self.findCommandRun:
@@ -661,47 +659,28 @@ class Script(default.Script):
         default.Script.onActiveDescendantChanged(self, event)
 
     def onChildrenChanged(self, event):
-        """Called when a child node has changed.
+        """Callback for object:children-changed accessibility events."""
 
-        Arguments:
-        - event: the Event
-        """
-
-        try:
-            anyDataRole = event.any_data.getRole()
-        except:
+        if self.utilities.isSpreadSheetCell(event.any_data):
+            orca.setLocusOfFocus(event, event.any_data)
             return
 
-        if anyDataRole == pyatspi.ROLE_TABLE:
-            if self.utilities.isSpreadSheetCell(event.any_data, True):
-                orca.setLocusOfFocus(event, event.any_data)
-            return
-
-        if anyDataRole == pyatspi.ROLE_TABLE_CELL:
+        if self.utilities.isLastCell(event.any_data):
             activeRow = self.pointOfReference.get('lastRow', -1)
             activeCol = self.pointOfReference.get('lastColumn', -1)
             if activeRow < 0 or activeCol < 0:
                 return
 
-            eventRow, eventCol, table = \
-                self.utilities.getRowColumnAndTable(event.any_data)
-            try:
-                itable = table.queryTable()
-            except NotImplementedError:
-                return
-
-            if eventRow == itable.nRows - 1 and eventCol == itable.nColumns - 1:
-                fullMessage = briefMessage = ""
-                voice = self.voices.get(settings.SYSTEM_VOICE)
-                if activeRow == itable.nRows:
-                    fullMessage = messages.TABLE_ROW_DELETED_FROM_END
-                    briefMessage = messages.TABLE_ROW_DELETED
-                else:
-                    fullMessage =  messages.TABLE_ROW_INSERTED_AT_END
-                    briefMessage = messages.TABLE_ROW_INSERTED
-                if fullMessage:
-                    self.presentMessage(fullMessage, briefMessage, voice)
-                return
+            self.utilities.handleUndoTextEvent(event)
+            rowCount, colCount = self.utilities.rowAndColumnCount(event.source)
+            if activeRow == rowCount:
+                full = messages.TABLE_ROW_DELETED_FROM_END
+                brief = messages.TABLE_ROW_DELETED
+            else:
+                full = messages.TABLE_ROW_INSERTED_AT_END
+                brief = messages.TABLE_ROW_INSERTED
+            self.presentMessage(full, brief)
+            return
 
         default.Script.onChildrenChanged(self, event)
 
@@ -715,6 +694,16 @@ class Script(default.Script):
         if self.utilities.isSameObject(orca_state.locusOfFocus, event.source):
             return
 
+        if self.utilities.isFocusableLabel(event.source):
+            orca.setLocusOfFocus(event, event.source)
+            return
+
+        if self.utilities.isZombie(event.source):
+            comboBox = self.utilities.containingComboBox(event.source)
+            if comboBox:
+                orca.setLocusOfFocus(event, comboBox, True)
+                return
+
         role = event.source.getRole()
 
         # This seems to be something we inherit from Gtk+
@@ -727,6 +716,11 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source)
             return
 
+        # Ditto.
+        if role == pyatspi.ROLE_TOGGLE_BUTTON:
+            orca.setLocusOfFocus(event, event.source)
+            return
+
         # Ditto.
         if role == pyatspi.ROLE_COMBO_BOX:
             orca.setLocusOfFocus(event, event.source)
@@ -743,24 +737,34 @@ class Script(default.Script):
         if self._inSayAll:
             return
 
-        if self.isStructuralNavigationCommand():
+        if self._lastCommandWasStructNav:
             return
 
         if not event.detail1:
             return
 
-        if event.source.getRoleName() == 'text frame':
+        if self.utilities.isAnInputLine(event.source):
+            msg = "SOFFICE: Event ignored: spam from inputLine"
+            debug.println(debug.LEVEL_INFO, msg, True)
             return
+        if event.source.childCount and self.utilities.isAnInputLine(event.source[0]):
+            msg = "SOFFICE: Event ignored: spam from inputLine parent"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+
+        role = event.source.getRole()
+        if role in [pyatspi.ROLE_TEXT, pyatspi.ROLE_LIST]:
+            comboBox = self.utilities.containingComboBox(event.source)
+            if comboBox:
+                orca.setLocusOfFocus(event, comboBox, True)
+                return
 
         parent = event.source.parent
-        if parent and parent.getRoleName() == 'text frame':
-            return
-
         if parent and parent.getRole() == pyatspi.ROLE_TOOL_BAR:
             default.Script.onFocusedChanged(self, event)
             return
 
-        role = event.source.getRole()
+        # TODO - JD: Verify this is still needed
         ignoreRoles = [pyatspi.ROLE_FILLER, pyatspi.ROLE_PANEL]
         if role in ignoreRoles:
             return
@@ -774,8 +778,7 @@ class Script(default.Script):
 
         if role == pyatspi.ROLE_PARAGRAPH:
             obj, offset = self.pointOfReference.get("lastCursorPosition", (None, -1))
-            textSelections = self.pointOfReference.get('textSelections', {})
-            start, end = textSelections.get(hash(obj), (0, 0))
+            start, end, string = self.utilities.getCachedTextSelection(obj)
             if start != end:
                 return
 
@@ -786,56 +789,32 @@ class Script(default.Script):
 
         # We should present this in response to active-descendant-changed events
         if event.source.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS):
-            if role != pyatspi.ROLE_LIST:
-                return
+            return
 
         default.Script.onFocusedChanged(self, event)
 
     def onCaretMoved(self, event):
-        """Called whenever the caret moves.
-
-        Arguments:
-        - event: the Event
-        """
-
-        if self.isStructuralNavigationCommand():
-            return
+        """Callback for object:text-caret-moved accessibility events."""
 
         if event.detail1 == -1:
             return
 
-        if self.utilities.isCellBeingEdited(event.source):
-            orca.setLocusOfFocus(event, event.source.parent, False)
-
-        if not orca_state.locusOfFocus \
-           or self.utilities.isZombie(orca_state.locusOfFocus):
-            orca.setLocusOfFocus(event, event.source)
-            default.Script.onCaretMoved(self, event)
-            return
-
-        if orca_state.locusOfFocus.getRole() == pyatspi.ROLE_TABLE_CELL:
-            default.Script.onCaretMoved(self, event)
-            return
-
-        # The lists and combo boxes in the Formatting toolbar emit
-        # object:active-descendant-changed events which cause us
-        # to set the locusOfFocus to the list item. If the user then
-        # arrows within the text portion, we will not present it due
-        # to the event not being from the locusOfFocus. A similar
-        # issue is present in the Target entry of the Hyperlink dialog
-        # for OOo 3.2.
-        #
-        if event.source.getRole() == pyatspi.ROLE_TEXT \
-           and self.utilities.ancestorWithRole(
-               event.source,
-               [pyatspi.ROLE_TOOL_BAR, pyatspi.ROLE_DIALOG],
-               [pyatspi.ROLE_FRAME]):
-            orca.setLocusOfFocus(event, event.source, False)
-
         if self.utilities._flowsFromOrToSelection(event.source):
+           return
+
+        if self._lastCommandWasStructNav:
             return
 
-        default.Script.onCaretMoved(self, event)
+        if self.utilities.isSpreadSheetCell(orca_state.locusOfFocus):
+            msg = "SOFFICE: locusOfFocus %s is spreadsheet cell" % orca_state.locusOfFocus
+            debug.println(debug.LEVEL_INFO, msg, True)
+
+            if not self.utilities.isCellBeingEdited(event.source):
+                msg = "SOFFICE: Event ignored: Source is not cell being edited."
+                debug.println(debug.LEVEL_INFO, msg, True)
+                return
+
+        super().onCaretMoved(event)
 
     def onCheckedChanged(self, event):
         """Callback for object:state-changed:checked accessibility events."""
@@ -866,11 +845,47 @@ class Script(default.Script):
         if weToggledIt:
             speech.speak(self.speechGenerator.generateSpeech(obj))
 
-    def onRowReordered(self, event):
-        """Callback for object:row-reordered accessibility events."""
+    def onSelectedChanged(self, event):
+        """Callback for object:state-changed:selected accessibility events."""
 
-        # We're seeing a crazy ton of these emitted bogusly.
-        pass
+        full, brief = "", ""
+        if self.utilities.isSelectedTextDeletionEvent(event):
+            msg = "SOFFICE: Change is believed to be due to deleting selected text"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            full = messages.SELECTION_DELETED
+        elif self.utilities.isSelectedTextRestoredEvent(event):
+            msg = "SOFFICE: Selection is believed to be due to restoring selected text"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            if self.utilities.handleUndoTextEvent(event):
+                full = messages.SELECTION_RESTORED
+
+        if full or brief:
+            self.presentMessage(full, brief)
+            self.utilities.updateCachedTextSelection(event.source)
+            return
+
+        super().onSelectedChanged(event)
+
+    def onSelectionChanged(self, event):
+        """Callback for object:selection-changed accessibility events."""
+
+        if not self.utilities.isComboBoxSelectionChange(event):
+            super().onSelectionChanged(event)
+            return
+
+        selectedChildren = self.utilities.selectedChildren(event.source)
+        if len(selectedChildren) == 1:
+            orca.setLocusOfFocus(event, selectedChildren[0], True)
+
+    def onTextSelectionChanged(self, event):
+        """Callback for object:text-selection-changed accessibility events."""
+
+        if self.utilities.isComboBoxNoise(event):
+            msg = "SOFFICE: Event is believed to be combo box noise"
+            debug.println(debug.LEVEL_INFO, msg, True)
+            return
+
+        super().onTextSelectionChanged(event)
 
     def getTextLineAtCaret(self, obj, offset=None, startOffset=None, endOffset=None):
         """To-be-removed. Returns the string, caretOffset, startOffset."""
@@ -913,5 +928,7 @@ class Script(default.Script):
     def onWindowDeactivated(self, event):
         """Callback for window:deactivate accessibility events."""
 
+        self._lastCommandWasStructNav = False
+
         super().onWindowDeactivated(event)
         self.spellcheck.deactivate()
