@@ -70,7 +70,7 @@ class Script(default.Script):
         self._lastCaretContext = None, -1
         self.sayAllOnLoadCheckButton = None
 
-        if _settingsManager.getSetting('sayAllOnLoad') == None:
+        if _settingsManager.getSetting('sayAllOnLoad') is None:
             _settingsManager.setSetting('sayAllOnLoad', True)
 
     def setupInputEventHandlers(self):
@@ -347,11 +347,7 @@ class Script(default.Script):
 
         phrase = self.utilities.substring(obj, startOffset, endOffset)
         if len(phrase) and phrase != "\n":
-            if phrase.isupper():
-                voice = self.voices[settings.UPPERCASE_VOICE]
-            else:
-                voice = self.voices[settings.DEFAULT_VOICE]
-
+            voice = self.speechGenerator.voice(string=phrase)
             phrase = self.utilities.adjustForRepeats(phrase)
             links = [x for x in obj if x.getRole() == pyatspi.ROLE_LINK]
             if links:
@@ -514,8 +510,10 @@ class Script(default.Script):
         if obj.getRole() == pyatspi.ROLE_LINK:
             obj = obj.parent
 
-        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
-        document = utils.findAncestor(obj, lambda x: x.getRole() in docRoles)
+        if self.utilities.isDocument(obj):
+            document = obj
+        else:
+            document = self.utilities.getContainingDocument(obj)
         if not document or document.getState().contains(pyatspi.STATE_BUSY):
             return
 
@@ -531,14 +529,16 @@ class Script(default.Script):
         if sayAllStyle == settings.SAYALL_STYLE_SENTENCE:
             boundary = pyatspi.TEXT_BOUNDARY_SENTENCE_START
 
+        voices = _settingsManager.getSetting('voices')
+        systemVoice = voices.get(settings.SYSTEM_VOICE)
+
         self._inSayAll = True
         offset = textObjs[0].queryText().caretOffset
         for textObj in textObjs:
             textSegments = self.getTextSegments(textObj, boundary, offset)
-            roleInfo = self.speechGenerator.getRoleName(textObj)
-            if roleInfo:
-                roleName, voice = roleInfo
-                textSegments.append([roleName, 0, -1, voice])
+            roleName = self.speechGenerator.getRoleName(textObj)
+            if roleName:
+                textSegments.append([roleName, 0, -1, systemVoice])
 
             for (string, start, end, voice) in textSegments:
                 context = speechserver.SayAllContext(textObj, string, start, end)
